/*
 * Copyright (C) 2021 Purism SPC
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
 */

#pragma once

#if !defined(_ADWAITA_INSIDE) && !defined(ADWAITA_COMPILATION)
#error "Only <adwaita.h> can be included directly."
#endif

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define ADW_TYPE_INSPECTOR_PAGE (adw_inspector_page_get_type())

G_DECLARE_FINAL_TYPE (AdwInspectorPage, adw_inspector_page, ADW, INSPECTOR_PAGE, GtkWidget)

G_END_DECLS
