/*
 * Copyright (C) 2025 GNOME Foundation Inc.
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 * Author: Alice Mikhaylenko <alicem@gnome.org>
 */

#pragma once

#include "adw-shortcut-label.h"

G_BEGIN_DECLS

void adw_shortcut_label_set_wrap (AdwShortcutLabel *self,
                                  gboolean          wrap);

G_END_DECLS
