dialog.shortcuts {
  preferencespage > scrolledwindow > viewport > clamp > box {
    border-spacing: 18px;

    preferencesgroup + preferencesgroup box.header {
      margin-top: 6px;
    }
  }

  .navigation-box {
    margin-top: -12px;

    > button {
      border-radius: 99px;
      padding: 3px 12px;
    }
  }

  .shortcut-row {
    padding: 9px 12px;
    min-height: 36px;

    .title-box {
      border-spacing: 3px;

      > .subtitle {
        @extend .caption;
        @extend .dimmed;
      }
    }
  }
}

shortcut-label {
  border-radius: 6px;
  @include focus-ring($offset: 1px, $outer: true);

  &, > box {
    border-spacing: 6px;
  }

  .keycap {
    background: color-mix(in srgb, currentColor 10%, transparent);
    box-shadow: inset 0 -2px var(--card-shade-color);

    @media (prefers-contrast: more) {
      box-shadow: inset 0 -2px var(--card-shade-color), 0 0 0 1px var(--border-color);
    }

    border-radius: 6px;
    padding: 6px;
    min-width: 20px;
  }

  > .dimmed {
    padding: 6px 0;
  }
}
