@keyframes spin {
  to { transform: rotate(1turn); }
}

spinner {
  background: none;
  opacity: 0; // non spinning spinner makes no sense
  -gtk-icon-source: -gtk-icontheme('process-working-symbolic');

  &:checked {
    opacity: 1;
    animation: spin 1s linear infinite;

    &:disabled { opacity: var(--disabled-opacity); }
  }
}

/**************
 * AdwSpinner *
 **************/

image.spinner:disabled {
  opacity: var(--disabled-opacity);
}
