#pragma once

#include <adwaita.h>

G_BEGIN_DECLS

#define ADW_TYPE_DEMO_PAGE_VIEW_SWITCHER (adw_demo_page_view_switcher_get_type())

G_DECLARE_FINAL_TYPE (AdwDemoPageViewSwitcher, adw_demo_page_view_switcher, ADW, DEMO_PAGE_VIEW_SWITCHER, AdwBin)

G_END_DECLS
