cursor-handle {
  all: unset;
  padding: 24px 20px;

  > contents {
    min-width: 20px;
    min-height: 20px;
    border-radius: 50%;

    background-color: var(--accent-bg-color);
  }

  &.top > contents {
    border-top-right-radius: 0;
  }

  &.bottom > contents {
    border-top-left-radius: 0;
    transform: translateX(1px);
  }

  &.insertion-cursor > contents {
    border-top-left-radius: 0;
    transform: translateX(1px) translateY(4px) rotate(45deg);
  }
}

magnifier {
  background-color: var(--view-bg-color);
}
