toolbarview.undershoot-top    popover scrolledwindow:not(.undershoot-top) undershoot.top,
toolbarview.undershoot-bottom popover scrolledwindow:not(.undershoot-bottom) undershoot.bottom {
  background: none;
  box-shadow: none;
}

popover {
  &.background {
    background-color: transparent;
    font: initial;
  }

  > arrow,
  > contents {
    background-color: var(--popover-bg-color);
    color: var(--popover-fg-color);
    background-clip: padding-box;
    border: 1px solid RGB(0 0 0 / 14%);
    box-shadow: 0 1px 5px 1px RGB(0 0 0 / 9%),
                0 2px 14px 3px RGB(0 0 0 / 5%);

    @media (prefers-contrast: more) {
      border: 1px solid $border_color;
    }
  }

  > contents {
    padding: 8px;
    border-radius: $popover_radius;

    > list,
    > .view,
    > toolbar {
      border-style: none;
      background-color: transparent;
    }
  }

  .solid-csd & {
    background: var(--popover-bg-color);
    color: var(--popover-fg-color);
    border: 1px solid var(--border-color);

    > arrow,
    > contents {
      box-shadow: none;
      border-radius: 0;
      background-clip: border-box;
      border: none;
    }
  }

  .osd &,
  &.touch-selection,
  &.magnifier {
    background-color: transparent;
    color: $osd_fg_color;

    > arrow,
    > contents {
      @extend %osd;

      border-color: RGB(255 255 255 / 10%);
      box-shadow: none;
    }
  }

  --shade-color: var(--popover-shade-color);
}
