#pragma once

#include <adwaita.h>

G_BEGIN_DECLS

#define ADW_TYPE_DEMO_PAGE_TOGGLES (adw_demo_page_toggles_get_type())

G_DECLARE_FINAL_TYPE (AdwDemoPageToggles, adw_demo_page_toggles, ADW, DEMO_PAGE_TOGGLES, AdwBin)

G_END_DECLS
