.docs-background {
  background: color-mix(in srgb, currentColor 5%, transparent);
  border-radius: 15px;
}

#opaque-button-1 {
  color: var(--success-fg-color);
  background-color: var(--success-bg-color);
}

#opaque-button-2 {
  color: var(--warning-fg-color);
  background-color: var(--warning-bg-color);
}

#vscroll {
  padding-top: 12px;
  padding-bottom: 12px;
}

#hscroll {
  padding-left: 12px;
  padding-right: 12px;
}

.overview-tab-page {
  background: linear-gradient(to bottom right, var(--blue-1) 25%, var(--green-1));
}

@media (prefers-color-scheme: dark) {
  .overview-tab-page {
    background: image(rgb(0 0 0 / 25%)),
                linear-gradient(to bottom right, var(--blue-5), var(--green-5) 80%);
  }
}

.tags button {
  border-radius: 99px;
  font-weight: normal;
}
