/* Public colors from Default */

@define-color theme_bg_color @window_bg_color;
@define-color theme_fg_color @window_fg_color;

@define-color theme_base_color @view_bg_color;
@define-color theme_text_color @view_fg_color;

@define-color theme_selected_bg_color @accent_bg_color;
@define-color theme_selected_fg_color @accent_fg_color;

@define-color insensitive_bg_color color-mix(in srgb, @window_bg_color 60%, @view_bg_color);
@define-color insensitive_fg_color color-mix(in srgb, @window_fg_color 50%, transparent);

@define-color insensitive_base_color @view_bg_color;

@define-color borders color-mix(in srgb, currentColor 15%, transparent);

@media (prefers-contrast: more) {
  @define-color borders color-mix(in srgb, currentColor 50%, transparent);
}

@define-color theme_unfocused_bg_color @window_bg_color;
@define-color theme_unfocused_fg_color @window_fg_color;

@define-color theme_unfocused_base_color @view_bg_color;
@define-color theme_unfocused_text_color @view_fg_color;

@define-color theme_unfocused_selected_bg_color @accent_bg_color;
@define-color theme_unfocused_selected_fg_color @accent_fg_color;

@define-color unfocused_insensitive_color @insensitive_bg_color;

@define-color unfocused_borders @borders;
