expander {
  min-width: 16px;
  min-height: 16px;
  -gtk-icon-source: -gtk-icontheme('pan-end-symbolic');
  &:dir(rtl) { -gtk-icon-source: -gtk-icontheme('pan-end-symbolic-rtl'); }

  &:disabled { filter: Opacity(var(--disabled-opacity)); }

  &:checked { -gtk-icon-source: -gtk-icontheme('pan-down-symbolic'); }
}

expander-widget {
  @include focus-ring("> box > title");

  > box > title {
    border-radius: 6px;

    > expander {
      opacity: .7;
    }

    &:hover > expander,
    &:active > expander {
      opacity: 1;
    }
  }
}
