paned {
  > separator {
    min-width: 1px;
    min-height: 1px;
    background: none;
    background-size: 1px 1px;

    &.wide {
      min-width: 5px;
      min-height: 5px;
    }
  }

  &.horizontal > separator {
    &:dir(ltr) {
      box-shadow: inset 1px 0 $border_color;
    }
    &:dir(rtl) {
      box-shadow: inset -1px 0 $border_color;
    }

    &.wide {
      box-shadow: inset  1px 0 $border_color,
                  inset -1px 0 $border_color;
    }
  }

  &.vertical > separator {
    box-shadow: inset 0 1px $border_color;

    &.wide {
      box-shadow: inset 0  1px $border_color,
                  inset 0 -1px $border_color;
    }
  }
}
